function [res_s] = F_multi_label_compute_scores(classReal_m, classFound_m, affinity_m)
% function [res_s] = F_multi_label_compute_scores(classReal_m, classFound_m, affinity_m)
%
% DESCRIPTION:
% ============
% compuytes the set of scores for multi-label classification
%
% INPUTS:
% =======
% - classReal_m		(nb_file, nb_class)
% - classFound_m	(nb_file, nb_class)
% - affinity_m		(nb_file, nb_class)
%
% OUTPUTS:
% ========
% - res_s	.recall_v		(nb_class, 1)
%			.recallMean
%
% Damien Tardieu (damien.tardieu@ircam.fr)
% Re-written by Geoffroy Peeters (peeters@ircam.fr) in 2012 for Media-Eval Music Tagging brave new task
%
% Copyright (c) 2012 IRCAM, All Rights Reserved.
% This work was founded by the ?Quaero? Program funded by Oseo French State agency for innovation.


nb_class = size(classReal_m, 2);

for num_class=1:nb_class
	res_s.byTAG(num_class).accuracy	= sum(classReal_m(:,num_class) == classFound_m(:,num_class)) / size(classFound_m,1);

	posP_v			= find(classReal_m(:,num_class));
	posN_v			= find(classReal_m(:,num_class)==0);
	tp_v(num_class) = sum(classFound_m(posP_v,num_class)==1);
	tn_v(num_class) = sum(classFound_m(posN_v,num_class)==0);
	fp_v(num_class) = sum(classFound_m(posN_v,num_class)==1);
	fn_v(num_class) = sum(classFound_m(posP_v,num_class)==0);
	
	res_s.byTAG(num_class).recall		= tp_v(num_class)/(tp_v(num_class)+fn_v(num_class));
	res_s.byTAG(num_class).precision	= tp_v(num_class)/(tp_v(num_class)+fp_v(num_class));
	res_s.byTAG(num_class).precision(isnan(res_s.byTAG(num_class).precision))=0;
	res_s.byTAG(num_class).specificity	= tn_v(num_class)./(tn_v(num_class)+fp_v(num_class));
	res_s.byTAG(num_class).fmeasure		= (2* res_s.byTAG(num_class).precision .* res_s.byTAG(num_class).recall)./(res_s.byTAG(num_class).precision+res_s.byTAG(num_class).recall);
	res_s.byTAG(num_class).fmeasure(isnan(res_s.byTAG(num_class).fmeasure))=0;
end



% ============================================
% === EXTRA-SCORES IF AFFINITY HAS BEEN COMPUTED
if nargin==3	
	nb_class = size(classReal_m, 2);
	% === Area Under ROC curve
	for num_class=1:nb_class
		[Xroc, Yroc, T, res_s.byTAG(num_class).auc] = perfcurve(classReal_m(:,num_class), affinity_m(:,num_class), 1);
	end
		% === Mean Average Precision
	for num_class=1:nb_class
		res_s.byTAG(num_class).map = Fmean_average_precision(classReal_m(:,num_class), affinity_m(:,num_class));
	end	
end
% ============================================







% ===============================
function [map] = Fmean_average_precision(class_v, affinity_v)
% Insprired by the writting made by Remi Foucard (remi.foucard@telecom-paristech.fr) Copyright 2011 Telecom Paristech
[sort_value_v, sort_pos_v] = sort(affinity_v, 'descend');

prec_v = zeros(length(class_v), 1);
for num_file=1:length(class_v);
	if class_v(num_file), prec_v(num_file) = mean(class_v(1:num_file)); end
end
map = sum(prec_v)/sum(class_v);
