function map = getMapinv(textFile)
% Returns a map object from an input text file.
% Every line in the text file should express a key-value pair for the map
% following the input format "[key]"\t"[value]"\n
% In the map, every key will be associated with a cell array of values
% corresponding to this key in the input text file.

f = fopen(textFile);

% retrieve keys and values (because of %q, the double quotes are needed)
data = textscan(f, '%q\t%q\n');
keys_in_data = data{2};
values_in_data = data{1};

% get unique keys for the map
unique_keys = unique(keys_in_data);
% pre-initialize values entries
values = cell(size(unique_keys));

for i = 1:length(unique_keys)
    current_key = unique_keys(i);
    % get all corresponding values 
    corresponding_values = values_in_data(find( ...
                             ismember(keys_in_data, current_key) == 1));
    values(i) = {corresponding_values};
end

map = containers.Map(unique_keys, values);