function [output] = Fmusiclef_audio(command, model_s, fileID, AUDIOFEATURE_DIR)
% This code is provided as the REFERENCE code to perform multi-label
% classification for the Media-Eval Music Tagging brave new task
%
% USAGE:
% =======
% - See REF_training_class for usage
%
% This code was partly written by Geoffroy Peeters (peeters@ircam.fr) in 2012 for the Media-Eval Music Tagging brave new task
% This work was partly supported by the Quaero Program funded by Oseo French State agency for innovation.


switch command
    
    case 'training'

        nb_file = length(model_s.filelist_c);
        for l=1:nb_file
            fprintf(1, '%d/%d\n', l, nb_file);            
            model_s.track_s(l) = FLtrain_onefile([AUDIOFEATURE_DIR filesep model_s.filelist_c{l} '.mp3.mfcc']);
        end        
        output = model_s;
        
        
    case 'classification'

        target_track_s = FLtrain_onefile([AUDIOFEATURE_DIR filesep fileID '.mp3.mfcc']);
        
        distance_v      = zeros(length(model_s.track_s), 1);
        for l=1:length(model_s.track_s)
            distance_v(l) = FL_symKL(target_track_s, model_s.track_s(l));
        end
        [min_value, min_pos]    = min(distance_v);
        
        % === return tag corresponding to the closest item
        output = model_s.taglist_c(find(model_s.matrix_m(min_pos,:)));

end


% ========================
function [track_s] = FLtrain_onefile(FILE)

data_m             = load(FILE);
% === data_m (nb_dim, nb_frame)
track_s.mu         = mean(data_m.')';
track_s.sig        = cov(data_m.');
track_s.det        = det(track_s.sig);
track_s.invsig     = inv(track_s.sig);


% ========================
function [distance] = FL_symKL(M0, M1)

distance = FL_KL(M0, M1) + FL_KL(M1,M0);

% ========================
function [distance] = FL_KL(M0, M1)

K           = length(M0.mu);
distance    = 1/2*( trace(M1.invsig*M0.sig) + (M1.mu-M0.mu)' * M1.invsig * (M1.mu-M0.mu) - log(M0.det/M1.det) - K );
