function [] = REF_training_classification(command)
% This code is provided as the REFERENCE code to perform multi-label
% classification for the Media-Eval Music Tagging brave new task
%
% USAGE:
% =======
% - First update the variables in the CONFIG part below
% - REF_training_classification('training') to perform training of the system
% - REF_training_classification('classification') to perform classification of the data
%
% This code was partly written by Geoffroy Peeters (peeters@ircam.fr) in 2012 for the Media-Eval Music Tagging brave new task
% This work was partly supported by the Quaero Program funded by Oseo French State agency for innovation.

% ============================
% === CONFIG
TRAIN_FILE          = '../data/train.csv';
TEST_FILE           = '../data/test.csv';
% audio from .mfcc files
% PLEASE CHANGE THE FOLLOWING TO THE PATH WHERE THE MFCC .CSV FILES RESIDE
AUDIOFEATURE_DIR    = 'C:/Research/Challenges/MultiMediaEval_2012/brave_new_music_task/release_data/audio_features/mfcc/';
% text features from matlab .mat
DATA_TEXT_MAT_FILE  = '../data/data-text.mat';
OUTPUT_MODEL_AUDIO  = '../data/data-model-audio.mat';
OUTPUT_MODEL_WEB  = '../data/data-model-web.mat';
OUTPUT_MODEL_LASTFM  = '../data/data-model-lastfm.mat';
OUTPUT_FUSED  = '../data/fused_results.mat';
OUTPUT_RESULTS      = '../data/result-classification.txt';
% ============================

% === LOAD TEXT FEATURE VECTORS (Web page-TFIDFs and last.fm tag-count-vectors)
text = load(DATA_TEXT_MAT_FILE);

switch command
    
    case 'training'
        
        % ===========================
        % === 1) READ GROUND-TRUTH DATA
        disp(['Loading data'])
        data_c = importdata(TRAIN_FILE);
        data_c = data_c.data;
        for l=1:length(data_c)
            pos = findstr(data_c{l}, char(9));
            file_c{l}= data_c{l}(1:pos-1);
            tag_c{l} = strrep(data_c{l}(pos:end), char(9), '');
        end
        
        model_s.filelist_c  = unique(file_c);
        model_s.taglist_c   = unique(tag_c);
        % === model_s.matrix (nb_file, nb_tag)
        model_s.matrix_m    = zeros(length(model_s.filelist_c), length(model_s.taglist_c));
        for f=1:length(model_s.filelist_c)
            pos_v = strmatch(model_s.filelist_c{f}, file_c, 'exact');
            for p=1:length(pos_v)
                pos = strmatch(tag_c{pos_v(p)}, model_s.taglist_c, 'exact');
                model_s.matrix_m(f,pos) = 1;
            end
        end
        
        % ===========================
        % === 2) TRAIN AUDIO
        disp(['Creating audio model'])
        audiomodel_s = Fmusiclef_audio('training', model_s, [], AUDIOFEATURE_DIR);
        
        % ===========================
        % === 3) TRAIN BY TEXT
        disp(['Creating web model'])
        webmodel_s = Fmusiclef_web('training', model_s, [], [], text);
        
        % ===========================
        % === 4) TRAIN BY LastFM-Tags
        disp(['Creating last.fm model'])
        lastfmmodel_s = Fmusiclef_lastfm('training', model_s, [], [], text);
        
        disp(['Storing models'])
        save(OUTPUT_MODEL_AUDIO, 'audiomodel_s', '-v7.3');
        save(OUTPUT_MODEL_WEB, 'webmodel_s', '-v7.3');
        save(OUTPUT_MODEL_LASTFM, 'lastfmmodel_s', '-v7.3');
        
        
    case 'classification'
        
        % ===========================
        % === READ TEST DATA
        disp(['Loading data'])
        data_c = importdata(TEST_FILE);
        data_c = data_c.data;
        
        % ===========================
        % === LOAD PREVIOUSLY TRAINED MODEL
        a = load(OUTPUT_MODEL_AUDIO); audiomodel_s = a.audiomodel_s;
        b = load(OUTPUT_MODEL_WEB); webmodel_s = b.webmodel_s;
        c = load(OUTPUT_MODEL_LASTFM); lastfmmodel_s = c.lastfmmodel_s;
        % store tags also for web model
        disp(['Classifying using audio, web, and last.fm tag data'])
         
        fused_results = zeros(length(data_c), length(audiomodel_s.taglist_c));
        file_ids = [];
        fid = fopen(OUTPUT_RESULTS, 'w');
        for l=1:length(data_c)
            fprintf(1, '%d/%d\n', l, length(data_c));
            
            fileID      = num2str(data_c(l,:));
            file_ids(end+1) = data_c(l,:);           % store file IDs
            % ===========================
            % === 1) CLASS BY AUDIO
            foundtag_audio_c  = Fmusiclef_audio('classification', audiomodel_s, fileID, AUDIOFEATURE_DIR);
            
            % ===========================
            % === 2) CLASS BY TEXT
            foundtag_web_c  = Fmusiclef_web('classification', audiomodel_s, webmodel_s, fileID, text);
            
            % ===========================
            % === 3) CLASS BY LastFM-Tags
            foundtag_lastfm_c  = Fmusiclef_lastfm('classification', audiomodel_s, lastfmmodel_s, fileID, text);
            
            % ===========================
            % fuse audio, web, and last.fm data
            for u=1:length(foundtag_audio_c)    % audio
                idx = strmatch(foundtag_audio_c(u), audiomodel_s.taglist_c, 'exact');
                fused_results(l,idx) = fused_results(l,idx)+1;
            end;
            for u=1:length(foundtag_web_c)  % web
                idx = strmatch(foundtag_web_c(u), audiomodel_s.taglist_c, 'exact');
                fused_results(l,idx) = fused_results(l,idx)+1;
            end;
            for u=1:length(foundtag_lastfm_c)   % last.fm
                idx = strmatch(foundtag_lastfm_c(u), audiomodel_s.taglist_c, 'exact');
                fused_results(l,idx) = fused_results(l,idx)+1;
            end;
            
            % === WRITE RESULTS TO FILE
            % code for writing all tags into results file (predictions by
            % all three classifiers)
%             for l=1:length(foundtag_audio_c)
%                 fprintf(fid, '%s\t%s\n', fileID, foundtag_audio_c{l});
%             end
%             for l=1:length(foundtag_web_c)
%                 fprintf(fid, '%s\t%s\n', fileID, foundtag_web_c{l});
%             end
%             for l=1:length(foundtag_lastfm_c)
%                 fprintf(fid, '%s\t%s\n', fileID, foundtag_lastfm_c{l});
%             end

            % perform late fusion: 
            % predict all tags on which 2 classifiers agree (if this is the
            % case); predict all tags for one classifier (if no agreement
            % on any tag)
            idx = find(fused_results(l,:)>=2);
            if isempty(idx),
                idx = find(fused_results(l,:)==1);
            end;                        
            tags = audiomodel_s.taglist_c(idx);
            % write tags
            for m=1:length(tags)
               fprintf(fid, '%s\t%s\n', num2str(file_ids(l)), tags{m});  
            end;
            
        end
        fclose(fid);
        
        save(OUTPUT_FUSED, 'fused_results', 'file_ids', '-v7.3');        
        
end









